
c A translation of fp2ccd.c into Fortran

c fp2ccd.c requires readcdf, readcof to read the camera definitions
c and set up the static variables.  These are now in readdewar.f


cccccccccccccccccccc
c polyvalue is from cosmos/source/cos_utils.c and computes a polynomial
c note that the number of coeffs is 1 greater than the degree of
c the polynomial because an 0th-order polynomial still has 1 term, etc.

      function polyvalue(x,coef,nord)

      real coef(nord+1)

      t=x
      val = coef(1)
      do i=1,nord
         val = val + t*coef(i+1)
         t = t*x
      end do
      polyvalue = val
      return
      end

cccccccccccccccccccc
c A translation of fp2ccd.c into Fortran, with a few changes
c including returning the chip number as an argument rather
c than the function value.

      subroutine fptochip(xfp,yfp,ichip,xccd,yccd)

      use dewdef

      parameter (IFDEBUG=0)
      real xccd,yccd,xfp,yfp
      integer ichip
      character*8 cam

      call initdewar()

      cam = 'SHORT'
      wave = 7500.
c pixels/mm
      pixscale = 66.67
c convert pixels to millimeters for the buffer around chips
      rpixbuf = 6./pixscale

c if we are taking input in pixels, convert to mm
      xfp2 = xfp/pixscale
      yfp2 = yfp/pixscale
c if input is in mm
c      xfp2 = xfp
c      yfp2 = yfp

      r = sqrt(xfp2**2 + yfp2**2)
      theta = acos(xfp2/r)
      if (yfp2 .lt. 0.) theta = -theta

c scale, distortion corrections
      if (cam .eq. 'SHORT') then
c short camera
c scale vs lambda
         dscl = dscale * (1.0 + 0.001 * polyvalue(wave,sl_coef,nsl_n))
c dtheta
         dt = 0.001*(polyvalue(r,sts1_coef,nsts1_n) *
     +        cos(theta - polyvalue(r,stt1_coef,nstt1_n)) +
     +        polyvalue(r,sts2_coef,nsts2_n) * 
     +        cos(2.*(theta - polyvalue(r,stt2_coef,nstt2_n))))
         theta = theta - dt
c dr
         drr = 0.001*(polyvalue(r,srs1_coef,nsrs1_n) *
     +        cos(theta - polyvalue(r,srt1_coef,nsrt1_n)) +
     +        polyvalue(r,srs2_coef,nsrs2_n) *
     +        cos(2.*(theta - polyvalue(r,srt2_coef,nsrt2_n))) +
     +        polyvalue(r,srz_coef,nsrz_n))
         r = r * (1.0-drr)
         if (IFDEBUG .ge. 1) write(*,*) "shortcam dscl,dt,drr ",
     $        dscl,dt,drr
      else
c long camera
         dscl = dscale
      end if

c use xfp2 and yfp2 to avoid changing the input values
      theta = theta - dcth
      xfp2 = r * cos(theta)
      yfp2 = r * sin(theta)
      xfp2 = xfp2 / dscl
      yfp2 = yfp2 / dscl
      xfp2 = xfp2 - dcx
      yfp2 = yfp2 - dcy

c identify chip
      ichip = -1
      i = 1
 10   continue
c Are the  xmin, xmax 0-based?  Allow a ~10 pixel buffer around a
c chips so pixels close to it get assigned to the right one.
      if (xfp2 .lt. xmin(i)-rpixbuf .or. xfp2 .gt. xmax(i)+rpixbuf .or. 
     +    yfp2 .lt. ymin(i)-rpixbuf .or. yfp2 .gt. ymax(i)+rpixbuf) then
         i = i + 1
         if (i .le. nchip) go to 10
      else
         ichip = i
      end if
      if (ichip .le. 0) then
c Not on any chip.  Here fp2ccd.c returns, but I think we want to
c keep going.  Maybe find the closest chip?  Just set it to 2 for now.
         ichip = 2
      end if

c transform coordinates
      xfp2 = xfp2 - xx0(ichip)
      yfp2 = yfp2 - yy0(ichip)
      r = sqrt(xfp2**2 + yfp2**2)
      theta = acos(xfp2/r)
      if (yfp2 .lt. 0.) theta = -theta
      theta = theta - dtheta(ichip)
      r = r * scale
      xfp2 = r*cos(theta)
      yfp2 = r*sin(theta)
      xccd = xz(ichip) + sxz(ichip)*xfp2   
      yccd = yz(ichip) + syz(ichip)*yfp2   
c Here fp2ccd.c checks the boundaries of the chip and sets chip|ichip=0
c if not in the boundaries, but I don't think we want to enforce that
c watch out for 0/1 base issues here
c      if(xccd .lt. 0 .or. xccd .gt. xchip .or. 
c     +     yccd .lt. 0 .or. yccd .gt. ychip) ichip = 0
      return
      end


cccccccccccccccccccc
c A translation of ccd2fp.c into Fortran

c      subroutine ccd2fp(ichip,xccd,yccd,wave,xfp,yfp)
c      subroutine ccd2fp(ichip,xccd,yccd,xfp,yfp)
      subroutine chiptofp(ichip,xccd,yccd,xfp,yfp)

      use dewdef

      parameter(IFDEBUG=0)
      integer ichip
      real xccd,yccd,wave,xfp,yfp
      character cam*8

      call initdewar()

      cam = 'SHORT'
c if we're not taking wavelength as input
      wave=7500.
c pixels/mm
      pixscale = 66.67

c      ichip0 = ichip-1
c convert to 8-ccd orientation
      xtrans = (xccd - xz(ichip)) / sxz(ichip)
      ytrans = (yccd - yz(ichip)) / syz(ichip)
c transform coords
      r = sqrt(xtrans**2 + ytrans**2)
      theta = acos(xtrans/r)
      if (ytrans .lt. 0.) theta = -theta
      theta = theta + dtheta(ichip)
      r = r /scale
      xtrans = r * cos(theta)
      ytrans = r * sin(theta)
      xtrans = xtrans + xx0(ichip)
      ytrans = ytrans + yy0(ichip)

c compute scale and r, theta offset for short camera
      if (cam .eq. 'SHORT') then
         dscl = dscale*(1.0 + 0.001*polyvalue(wave,sl_coef,nsl_n))
c need to do dtheta and drr, which depend on r and theta in the fp,
c so difficult to invert.  See below.
c
      else
c long camera
         dcsl = dscale
      end if
      if (IFDEBUG .ge. 2) 
     $     write (*,*) "step1 ",r,theta,xtrans,ytrans,dscale,dscl
         
c camera offsets
      xtrans = xtrans + dcx
      ytrans = ytrans + dcy
      xtrans = xtrans * dscl
      ytrans = ytrans * dscl
      r = sqrt(xtrans**2 + ytrans**2)
      theta = acos(xtrans/r)
      if (ytrans .lt. 0.) theta = -theta
      theta = theta + dcth
      if (IFDEBUG .ge. 2) 
     $     write (*,*) "step2 ",r,theta,xtrans,ytrans

c now we compute dtheta and drr, hoping that the corrections are
c small so that we can use the same formula to reverse the correction
c so we compute the reverse correction and change the operation
c on theta or r to the inverse
      if (cam .eq. 'SHORT') then
c dtheta
         dt = 0.001*(polyvalue(r,sts1_coef,nsts1_n) *
     +        cos(theta - polyvalue(r,stt1_coef,nstt1_n)) +
     +        polyvalue(r,sts2_coef,nsts2_n) * 
     +        cos(2.*(theta - polyvalue(r,stt2_coef,nstt2_n))))
         theta = theta + dt
c dr
         drr = 0.001*(polyvalue(r,srs1_coef,nsrs1_n) *
     +        cos(theta - polyvalue(r,srt1_coef,nsrt1_n)) +
     +        polyvalue(r,srs2_coef,nsrs2_n) *
     +        cos(2.*(theta - polyvalue(r,srt2_coef,nsrt2_n))) +
     +        polyvalue(r,srz_coef,nsrz_n))
         r = r / (1.0-drr)
         if (IFDEBUG .ge. 1) write(*,*) "shortcam dscl,dt,drr ",
     $        dscl,dt,drr
      end if
c returned values
      xfp = r * cos(theta)
      yfp = r * sin(theta)
c if we are outputting in pixels rather than mm
      xfp = xfp * pixscale
      yfp = yfp * pixscale
      if (IFDEBUG .ge. 2) 
     $     write (*,*) "step2 ",r,theta,xfp,yfp
      return
      end
