
c  mrqfit.f 
c  nonlinear least squares fitting
c    based on the Numerical Recipes routine mrqmin
c    Two major changes:
c    1 - mrqfit provides a little front end to do the
c        iteration calling mrqmin
c    2 - gaussj is modified to avoid crashing when it finds
c        a singular matrix.  Instead it reports a failure in the
c        integer flag "ier"

c  "ier" is added to names of the NumRec routines to avoid
c   collisions when compiled with NumRec library.  Though
c   actually only mrqmin and gaussj are modified

      subroutine mrqfit(x,y,yerr,npts,fitpars,ifit,nfit,
     $                  covar,alpha,ncovar,chisq,fitfunc,ifconv)

      real x(*),y(*),yerr(*)
      real fitpars(nfit)
      real covar(ncovar,ncovar),alpha(ncovar,ncovar)
c      logical ifconv
      integer ifconv,npts

      external fitfunc

c      print *,'entering mrqfit'
      ier=0
      alamda = -1.
      itmax = 100
      conv = 1.e-3
      ochisq = 1000.

      ifconv = 1
      do iter = 1,itmax
         oldalam = alamda

c         print *, 'calling mrqmin',iter

         call mrqminier(x,y,yerr,npts,fitpars,ifit,nfit,
     $     covar,alpha,ncovar,chisq,fitfunc,alamda,ier)

         if(ier .ne. 0) then 
            ifconv = 0
            go to 666
         endif
            
c         print *,'chisq= ', chisq

         dchisq = (ochisq - chisq) / chisq
         ochisq = chisq
c         write(*,*) iter,chisq,dchisq,pars(1),pars(2)
         if ((dchisq .lt. conv) .and. (oldalam .gt. alamda)) go to 777
      end do

 666  continue
      if (ifconv.eq.0) write (*,'(a,2(a,i4.0),a)')
     $     "WARNING: Singular matrix in GAUSSJ."
     $     ,"  Exiting during iteration ",iter-1," of ",itmax,"."

c     Jump here if we converged
 777  continue
      alamda = 0.
      call mrqminier(x,y,yerr,npts,fitpars,ifit,nfit, covar,alpha,ncovar
     $     ,chisq,fitfunc,alamda,ier)

      return
      end

      SUBROUTINE mrqminier(x,y,sig,ndata,a,ia,ma,covar,alpha,nca,chisq,
     *funcs,alamda,ier)
      INTEGER ma,nca,ndata,ia(ma),MMAX,ier
      REAL alamda,chisq,funcs,a(ma),alpha(nca,nca),covar(nca,nca),
     *sig(ndata),x(ndata),y(ndata)
      PARAMETER (MMAX=20)
CU    USES covsrt,gaussj,mrqcof
      INTEGER j,k,l,m,mfit
      REAL ochisq,atry(MMAX),beta(MMAX),da(MMAX)
      SAVE ochisq,atry,beta,da,mfit
      ier=0
      if(alamda.lt.0.)then
        mfit=0
        do 11 j=1,ma
          if (ia(j).ne.0) mfit=mfit+1
11      continue
        alamda=0.001
        call mrqcof(x,y,sig,ndata,a,ia,ma,alpha,beta,nca,chisq,funcs)
        ochisq=chisq
        do 12 j=1,ma
          atry(j)=a(j)
12      continue
      endif
      j=0
      do 14 l=1,ma
        if(ia(l).ne.0) then
          j=j+1
          k=0
          do 13 m=1,ma
            if(ia(m).ne.0) then
              k=k+1
              covar(j,k)=alpha(j,k)
            endif
13        continue
          covar(j,j)=alpha(j,j)*(1.+alamda)
          da(j)=beta(j)
        endif
14    continue
      call gaussjier(covar,mfit,nca,da,1,1,ier)
c      if (ier .ne. 0) return
      if(alamda.eq.0.)then
        call covsrt(covar,nca,ma,ia,mfit)
        return
      endif
      j=0
      do 15 l=1,ma
        if(ia(l).ne.0) then
          j=j+1
          atry(l)=a(l)+da(j)
        endif
15    continue
      call mrqcof(x,y,sig,ndata,atry,ia,ma,covar,da,nca,chisq,funcs)
      if(chisq.lt.ochisq)then
        alamda=0.1*alamda
        ochisq=chisq
        j=0
        do 17 l=1,ma
          if(ia(l).ne.0) then
            j=j+1
            k=0
            do 16 m=1,ma
              if(ia(m).ne.0) then
                k=k+1
                alpha(j,k)=covar(j,k)
              endif
16          continue
            beta(j)=da(j)
            a(l)=atry(l)
          endif
17      continue
      else
        alamda=10.*alamda
        chisq=ochisq
      endif
      return
      END



      SUBROUTINE mrqcof(x,y,sig,ndata,a,ia,ma,alpha,beta,nalp,chisq,
     *funcs)
      INTEGER ma,nalp,ndata,ia(ma),MMAX
      REAL chisq,a(ma),alpha(nalp,nalp),beta(ma),sig(ndata),x(ndata),
     *y(ndata)
      EXTERNAL funcs
      PARAMETER (MMAX=20)
      INTEGER mfit,i,j,k,l,m
      REAL dy,sig2i,wt,ymod,dyda(MMAX)
      mfit=0
      do 11 j=1,ma
        if (ia(j).ne.0) mfit=mfit+1
11    continue
      do 13 j=1,mfit
        do 12 k=1,j
          alpha(j,k)=0.
12      continue
        beta(j)=0.
13    continue
      chisq=0.
      do 16 i=1,ndata
        call funcs(x(i),a,ymod,dyda,ma)
        sig2i=1./(sig(i)*sig(i))
        dy=y(i)-ymod
        j=0
        do 15 l=1,ma
          if(ia(l).ne.0) then
            j=j+1
            wt=dyda(l)*sig2i
            k=0
            do 14 m=1,l
              if(ia(m).ne.0) then
                k=k+1
                alpha(j,k)=alpha(j,k)+wt*dyda(m)
              endif
14          continue
            beta(j)=beta(j)+dy*wt
          endif
15      continue
        chisq=chisq+dy*dy*sig2i
16    continue
      do 18 j=2,mfit
        do 17 k=1,j-1
          alpha(k,j)=alpha(j,k)
17      continue
18    continue
      return
      END


      SUBROUTINE covsrt(covar,npc,ma,ia,mfit)
      INTEGER ma,mfit,npc,ia(ma)
      REAL covar(npc,npc)
      INTEGER i,j,k
      REAL swap
      do 12 i=mfit+1,ma
        do 11 j=1,i
          covar(i,j)=0.
          covar(j,i)=0.
11      continue
12    continue
      k=mfit
      do 15 j=ma,1,-1
        if(ia(j).ne.0)then
          do 13 i=1,ma
            swap=covar(i,k)
            covar(i,k)=covar(i,j)
            covar(i,j)=swap
13        continue
          do 14 i=1,ma
            swap=covar(k,i)
            covar(k,i)=covar(j,i)
            covar(j,i)=swap
14        continue
          k=k-1
        endif
15    continue
      return
      END



c  gaussjier is due to Povilas?  Slightly modified to avoid
c  crashing due to singular matrix - instead it reports it 
c  and returns ier=1 (otherwise ier=0)

      SUBROUTINE gaussjier(a,n,np,b,m,mp,ier)
      INTEGER m,mp,n,np,NMAX,ier
      REAL a(np,np),b(np,mp)
      PARAMETER (NMAX=50)
      INTEGER i,icol,irow,j,k,l,ll,indxc(NMAX),indxr(NMAX),ipiv(NMAX)
      REAL big,dum,pivinv
      ier=0
      do 11 j=1,n
        ipiv(j)=0
11    continue
      do 22 i=1,n
        big=0.
        do 13 j=1,n
          if(ipiv(j).ne.1)then
            do 12 k=1,n
              if (ipiv(k).eq.0) then
                if (abs(a(j,k)).ge.big)then
                  big=abs(a(j,k))
                  irow=j
                  icol=k
                endif
              else if (ipiv(k).gt.1) then
                print *, 'singular matrix in gaussj'
                ier=1
                return
              endif
12          continue
          endif
13      continue
        ipiv(icol)=ipiv(icol)+1
        if (irow.ne.icol) then
          do 14 l=1,n
            dum=a(irow,l)
            a(irow,l)=a(icol,l)
            a(icol,l)=dum
14        continue
          do 15 l=1,m
            dum=b(irow,l)
            b(irow,l)=b(icol,l)
            b(icol,l)=dum
15        continue
        endif
        indxr(i)=irow
        indxc(i)=icol
        if (a(icol,icol).eq.0.) then
           print *, 'singular matrix in gaussj'
           ier=1
           return
        endif
        pivinv=1./a(icol,icol)
        a(icol,icol)=1.
        do 16 l=1,n
          a(icol,l)=a(icol,l)*pivinv
16      continue
        do 17 l=1,m
          b(icol,l)=b(icol,l)*pivinv
17      continue
        do 21 ll=1,n
          if(ll.ne.icol)then
            dum=a(ll,icol)
            a(ll,icol)=0.
            do 18 l=1,n
              a(ll,l)=a(ll,l)-a(icol,l)*dum
18          continue
            do 19 l=1,m
              b(ll,l)=b(ll,l)-b(icol,l)*dum
19          continue
          endif
21      continue
22    continue
      do 24 l=n,1,-1
        if(indxr(l).ne.indxc(l))then
          do 23 k=1,n
            dum=a(k,indxr(l))
            a(k,indxr(l))=a(k,indxc(l))
            a(k,indxc(l))=dum
23        continue
        endif
24    continue
      return
      END

