C-----------------------------------------------------------C
C     lines.f
C     
C     Description: 
C     lines.f contains a number of subroutines that are needed
C     by autosausage.f for creating an artificial spectra 
C     to calibrate new sausages taken with the MMTF.
C
C     Please check autosausage.f for more details on calibrating
C     sausages.
C
C     Authors
C     Written by: Benjamin Shaya
C     In association with Mike McDonald and Sylvain Veilleux
C     For use with the Maryland-Magellan Tunable Filter (MMTF)
C
C      History:
C  7/2010 - Created 
C     (Ben, et al.,University of Maryland, College Park)
C-----------------------------------------------------------C

C     calculates the intensity at a given z coordinate using
C     numOrd orders of the lines found in lines(:), using
C     a and b to transform from wavelength space to z space.
      function getI(a,b,numOrd,z,w,numlines
     $  ,lines,scl,base)
        double precision a(4),b(4),z,w,base
        double precision lines(10),scl(10),getI
        integer numOrd,numlines,i,j

        getI=0

C       sum the intensities at the wavelengths that would be 
C       transformed to the requested z coordinate using the
C       a and b values
        do i=1,numOrd
          getI = getI+spec(z*b(i)+a(i),w,numLines,lines,scl,b(i))
        end do
        getI = getI-base
      end function getI

C     Finds the intensity of the artificial lines at a given
C     wavelength.
      function spec(wave,w,lines,line,scl,b)
        double precision wave,spec, gauss,line(10),scl(10)
        double precision b,w
        integer lines,i

        spec = 0
        do i=1,lines
          spec = spec+scl(i)*cauchy(wave,line(i),w*b)
        end do

      end function spec

C     Calculate the intensity of a Gaussian curve at point x
C     with FWHM w and centered at u.
      function gauss(x,u,w)
        double precision x,u,w,s,gauss

        s = w / 2. * sqrt(2. * log(2.))
        gauss = dexp( -1.*((x-u) ** 2) / (2 * s **2))
      end function gauss

C     Calculate the intensity of a Lorrentzian curve at point x
C     with FWHM w and centered at u.
      function cauchy(x,u,w)
        double precision x,u,w,cauchy

        cauchy= w*w / ((x-u)**2 + w*w)
      end function cauchy

C     This subroutine contains all data for lines visible in
C     each filter/z_c.  Data can be requested by specifying
C     a filter and z_c (z in this subroutine).
C     For notes on how to add additional filters or z_c, scroll to the
C     bottom of this file.
      subroutine specDat(z,filt,w,lines,line,scl,a,b,n,good)

        implicit none

        integer z,lines
        character(80)filt
        double precision w,line(10),scl(10),a,b,n,good
        logical ignorelow

        if(z.ne.-2 .and.z.ne.1 .and.z.ne.3)then
          print*,"Unrecognized z coarse setting."
          stop
        end if
        if(index(filt,"5100") .gt. 0)then
          if(z .eq. -2) then

c            write(*,'(a,"\n",a,"\n",a)')
c     $      "Warning: The calibration profile for this filter+z_c",
c     $      "Contain lines that were guesses.",
c     $      "Use at own risk."
c           Ne 5080.38 [27.5]
c           Ne 5116.50 [27.5]
c           Ne 5144.93 [27.5]  + Ne 5145.01 [27.5]
c           Ne 5037.59 [28.5] (+ Ne 5031.34 [28.5])

            a=4357.5
            b=0.33901
            n=27.75
            w=40
            lines=6
            line(1)=5080.38
            line(2)=5116.50
            line(3)=5144.93
            line(4)=5037.75
            line(5)=5052.9414
            line(6)=5188.6122
            line(7)=5005.1587

            scl(1)=.53
            scl(2)=1.
            scl(3)=.4
            scl(4)=.36
            scl(5)=.3
            scl(6)=.2
            scl(7)=.1

            good=.08
          else if (z.eq.1)then

c           Ne 5144.93 [37.7]  + Ne 5145.01 [37.7]
c           Ne 5037.59 [38.7] (+ Ne 5031.34 [38.7] + Ne 5188.61 [37.7])
c           Ne 5080.38 [38.7]
c           Ne 5116.50 [38.7]

            a=4607.27
            b=0.25681
            n=37.09
            w=40
            Lines=7
            line(1)=5144.93
            line(2)=5037.59
            line(3)=5080.38
            line(4)=5116.50
            line(5)=5188.61
            line(6)=4980
            line(7)=5151.9610 
            scl(1)=.4
            scl(2)=.6
            scl(3)=.67
            scl(4)=1.2
            scl(5)=.32
            scl(6)=.0
            scl(7)=.2
            good=.17
          else if (z.eq.3)then

c           write(*,'(3(a,"\n"),a)')
c     $      "Warning: The calibration profile for this filter+z_c",
c     $      "Contain weak lines that were guesses:",
c     $      "5074.59, 5083.9773, 5150.08, 5151.96, 5090.321",
c     $      "Use at own risk."
c          Ne 5080.38 [46.6] (+ Ne 5188.61 [45.6])
c          Ne 5116.50 [46.6]
c          Ne 5144.93 [46.6]  + Ne 5145.01 [46.6] + Ne 5037.59 [47.6] (+ Ne 5031.34 [47.6])

            a=4678.24
            b=0.19935
            n=47.71
            w=40
            Lines=10
            line(1)=5080.38
            line(2)=5116.50
            line(3)=5144.93
            line(4)=5031.34
            line(5)=5074.5971
            line(6)=5151.9610
            line(7)=5150.0842
            line(8)=5083.9773
            line(9)=5037.59
            line(10)=5090.321
            scl(1)=.34
            scl(2)=1
            scl(3)=.42
            scl(4)=.1
            scl(5)=.2
            scl(6)=.10
            scl(7)=.4
            scl(8)=.2
            scl(9)=.1
            scl(10)=.1
            good = -1
          end if
        else if (index(filt,"5300") .gt. 0) then
          if(z .eq. -2) then

c           Ne 5330.78 [25.7]  + Ne 5341.09 [25.7]
c           Ne 5193.22 [26.7] (+ Ne 5400.56 [25.7])

            a=4521.61
            b=0.36948
            n=25.38
            w=31*.7
            Lines=6
            line(1)=5330.78
            line(2)=5341.09
            line(3)=5193.22
            line(4)=5400.56
            line(5)=5214.3389
            line(6)=5234.0271
            scl(1)=.3
            scl(2)=.9
            scl(3)=.3
            scl(4)=.0
            scl(5)=.08
            scl(6)=.06
            good = -1
          else if (z.eq.1)then

c           Ne 5330.78 [34.4] + Ne 5341.09 [34.4] 
c           Ne 5400.56 [34.4]

            ignorelow = .false.

            a=4719.06
            b=0.27509
            n=34.36
            w=31*.7
            Lines=4
            line(1)=5330.78
            line(2)=5341.09
            line(3)=5400.56
            line(4)=5358
            scl(1)=.7
            scl(2)=.83
            scl(3)=.4
            scl(4)=.08
            good = .03
          else if (z.eq.3)then

c           Ne 5330.78 [42.3] + Ne 5341.09 [42.3]
c           Ne 5400.56 [42.3]

            a=4751.83
            b=0.22143
            n=42.89
            w=31*.7
            lines=3
            line(1)=5330.78
            line(2)=5341.09
            line(3)=5400.56
            scl(1)=.8
            scl(2)=.9
            scl(3)=.4
            good=.04
          end if
        else if(index(filt,"6400") .gt. 0)then
          if(z .eq. -2) then
            print*,"6400n2"
c           Ne 6334.4276 [20.39]
c           Ne 6382.9914 [20.39] + Ne 6402.248 [20.39]

            a = 5416.3
            b =0.45919
            n =20.5
            w = 25
            lines=5
            line(1)=6334.4276
            line(2)=6382.9914
            line(3)=6402.248
            line(4)=6506.5277
            line(5)=6532.8824

            scl(1)=.35
            scl(2)=.35
            scl(3)=.9
            scl(4)=.3
            scl(5)=.1
            good=.03
          else if(z .eq. 1) then
c           Ne 6334.4276 [30.41]
c           Ne 6382.9914 [30.41] + Ne 6402.248 [30.41]

            a=5735.49
            b=0.31991
            n=29.57
            w=25
            lines = 4
            line(1)=6334.4276
            line(2)=6382.9914
            line(3)=6402.248
            line(4)=6506.5277
            line(5)=6304.7889
            scl(1)=.4
            scl(2)=.3
            scl(3)=1
            scl(4)=.3
            scl(5)=.1
            good=.04
          else if(z .eq. 3) then
            print*,"No data for this z_c and filter"
            stop
          end if
        else if(index(filt,"6600") .gt. 0) then
          if(z .eq. -2) then
c           Ne 6678.27 [20.00]
c           Ne 6402.24 [21.00] (+ 6717.04 [20.00])
c           Ne 6506.52 [21.00]
c           Ne 6532.88 [21.00]
c           Ne 6598.95 [21.00]

            a = 5663.93
            b =0.48381
            n =19.59

            w = 25
            lines=8
            line(1)=6678.27
            line(2)=6402.24
            line(3)=6506.52
            line(4)=6532.88
            line(5)=6598.95
            line(6)=6423.7044
            line(7)=6304.7893
            line(8)=6382.9914

            scl(1)=.38
            scl(2)=1
            scl(3)=0.7
            scl(4)=.42
            scl(5)=.39
            scl(6)=.2
            scl(7)=.3
            scl(8)=.3
            good=.07

          else if (z.eq.1) then
c           Ne 6678.27 [29.15]
c           Ne 6506.52 [30.15] (+ 6717.04 [29.15])
c           Ne 6532.88 [30.15]
c           Ne 6598.95 [30.15]
c           Ne 6402.24 [31.15]
c           Ne 6678.27 [30.15]

            a = 5946.5786
            b = 0.3229
            n = 29.25
            w=25
            lines=6
            line(1)=6678.27
            line(2)=6506.52
            line(3)=6717.04
            line(4)=6532.88
            line(5)=6598.95
            line(6)=6402.24
            scl(1)=.45
            scl(2)=.95
            scl(3)=0.1
            scl(4)=.4
            scl(5)=.48
            scl(6)=.95
            good=.07
          else if (z .eq. 3) then

c           Ne 6402.24 [36.13]
c           Ne 6598.95 [35.13]
c           Ne 6678.27 [35.13]
c           Ne 6506.52 [36.13]
c           Ne 6532.88 [36.13] (+ 6717.04 [35.13])

            a = 6025.6
            b = 0.27277
            n = 34.78
            w=25
            lines=5
            line(1)=6402.24
            line(2)=6598.95
            line(3)=6678.27
            line(4)=6506.52
            line(5)=6532.88
            scl(1)=.8
            scl(2)=.5
            scl(3)=.4
            scl(4)=1.
            scl(5)=.45
            good=.05
          end if
        else if(index(filt,"6800") .gt. 0) then
          if(z .eq. -2) then

c  Ne 6929.47 [19.3]
c  Ne 6678.28 [20.3] + Ne 6717.04 [20.3] (+ Ne 7032.41 [19.3])

            a=6063.12
            b=0.45652
            n=20.67
            w=25
            lines = 3
            line(1)=6929.47
            line(2)=6678.28
            line(3)=6717.04
            line(4)=7032.41
            scl(1)=1.
            scl(2)=.3
            scl(3)=.6
            scl(4)=.1
            good=.06
          else if(z .eq. 1) then

            a =6443.374
            b = 0.3040
            n = 31.06

c           Ne 6929.47 [31.1]
c           Ne 6738.03 [32.1]
c           Ne 7024.05 [31.1] + 7032.41 [31.1]
c           Ne 6929.47 [32.1]
  
            w = 25
            lines=4
            line(1)=6929.47
            line(2)=6738.03
            line(3)=7024.05
            line(4)=7032.41
            scl(1)=1
            scl(2)=.53
            scl(3)=.12
            scl(4)=.09
            good=.03
          else if(z .eq. 3) then

c            print*,"Warning: some of the lines in this profile "//
c     $"(7064.762, 6526.03)"
c            print*,"are guesses."
c            print*,"Use with caution."
c           Ne 6738.03 [38.8]
c           Ne 6929.47 [37.8]
c           Ne 7032.41 [37.8]
c           Ne 

            a = 6606.12
            b = 0.27073
            n = 37.02

            w = 25
            lines=5
            line(1)=6738.03
            line(2)=6929.47
            line(3)=7032.41
            line(4)=6886.94
            line(5)=7067.724
            scl(1)=.48
            scl(2)=.95
            scl(3)=.12
            scl(4)=.2
            scl(5)=.1
            good=.04
          end if
        else if(index(filt,"7050") .gt. 0) then
          if(z .eq. -2) then

c           Ar 6965.431 [23.8] + ???
c           Ar 7067.218 [23.8]
c           Ar 7147.042 [23.8]

            a=6086.72
            b=0.44120
            n=21.35

            w = 25
            lines = 3
            line(1) = 6965.431
            line(2) = 7067.218
            line(3) = 7147.042
            scl(1) = 1.
            scl(2) = .3
            scl(3) = .03
            good=.02

          else if (z.eq. 1)then

            a = 6448.2
            b = 0.31685
            n = 29.95

c           Ar 7067.218 [22.8]
c           Ar 7147.042 [22.8]
c           Ar 6965.431 [23.8] + ???

            w = 25
            lines=3
            line(1)=6965.431
            line(2)=7067.218
            line(3)=7147.042
            scl(1)=1.
            scl(2)=.35
            scl(3)=.03
            good=.05
          else if (z.eq. 3)then
c           Ar 6965.431 [23.8] + ???
c           Ar 7067.218 [23.8]
c           Ar 7147.042 [23.8]

            a=6437.37
            b=0.26691
            n=35.58
            w=25
            lines = 3
            line(1)=6965.431
            line(2)=7067.218
            line(3)=7147.042
            scl(1)=1.
            scl(2)=.35
            scl(3)=.03
            good=.02
          end if
        else        if(index(filt,"8200") .gt. 0)then
          if(z .eq. -2) then

c           Kr 8190.06 [23.8]
c           Xe 8231.63 [23.8]
c           Kr 8112.90 [24.8]

            a=7430.71
            b=0.39708
            n=23.76
            w=40
            lines=3
            line(1)=8190.06
            line(2)=8231.63
            line(3)=8112.90
            scl(1)=.3
            scl(2)=.1
            scl(3)=1
            good= 0.02

          else if (z.eq.1)then

c           Kr 8190.06 [23.8]
c           Xe 8231.63 [23.8]
c           Kr 8112.90 [24.8]

            a=7631.39
            b=0.30778
            n=30.73
            w=40
            lines=3
            line(1)=8190.06
            line(2)=8231.63
            line(3)=8112.90
            scl(1)=.3
            scl(2)=.1
            scl(3)=1
            good= 0.02
          else if (z.eq.3)then



c           Kr 8190.06 [23.8]
c           Xe 8231.63 [23.8]
c           Kr 8112.90 [24.8]


            a=7651.45
            b=0.26494
            n=35.72
            w=40
            lines=3
            line(1)=8190.06
            line(2)=8231.63
            line(3)=8112.90
            scl(1)=.3
            scl(2)=.1
            scl(3)=1
            good=.03

          end if
        else if (index(filt,"9150") .gt. 0) then
          if(z .eq. -2) then

c           Xe 8952.78   [22]
c           Xe 9045.4466 [22]
c           Xe 9162.6520 [22]


            a=8129.42
            b=0.43313
            n=21.76
            w=40
            Lines=3
            line(1)=8952.78
            line(2)=9045.4466
            line(3)=9162.6520
            scl(1)=.07
            scl(2)=.58
            scl(3)=1
            good=0.02

          else if (z.eq.1)then

c           Xe 9045.4466 [29]
c           Xe 9162.6520 [29]
c           Xe 8952.78   [30]

            a=8186.04
            b=0.32597
            n=29.13
            w=40
            Lines=3
            line(1)=9045.4466
            line(2)=9162.6520
            line(3)=8952.78
            scl(1)=.6
            scl(2)=1
            scl(3)=.1
            good=0.03

          else if (z.eq.3)then

c           Xe 8952.78   [33]
c           Xe 9045.4466 [33]
c           Xe 9162.6520 [33]

            a=8370.21
            b=0.28638
            n=33.17
            w=40
            lines=3
            line(1)=8952.78
            line(2)=9045.4466
            line(3)=9162.6520
            scl(1)=.1
            scl(2)=.6
            scl(3)=1
            good=0.024

          end if
        else
          print*,"Unrecognized filter."
          stop
        end if
        if(good.gt. 0) good = dmax1(good,dble(.02))
      end subroutine specDat

C     ADDING ADDITIONAL DATA TO specDat:

C     To add more settings to this file, you simply need to specify
C      *number of lines ("lines")
C        *If you need to have more than 10 lines, you will need to
C         increase the size of the lines array in all functions
C         and subroutines here and in autosausage.f
C      *the wavelength of each line ("line(#)")
C      *the relative intensity of each line ("scl(#)")
C      *the width in z space of each line ("W")
C      *Guess a, b, and n values for the calibration.
C         *a must be within 500 of correct, b within .2, and n within 10
C      *The RMS value autosausage will find in a good fit for that setting
C         ("good").  This can be set to a negative number to disable
C         the estimated goodness of fit feature for that setup.
C
C     For now, good is set to be at least .02 to avoid throwing away
C     perfectly acceptable results.
C     This should probably be further constrained.

C     If adding a new filter, be sure update autosausage.f so it will 
C     recognize the filter (function getProperFilter)

C     If adding a new z_coarse setting, be sure to modify the
C     "if" statement at the start of specDat so it does not immediately
C     reject the setting.
        
