
c     Description
c
c     Use gravity angle, X_f and Y_f from reference images to compute the
c     new x and y parallelism based on the new gravity angle.
c
c     Authors
c
c     Michael McDonald
c
c     History
c
c     08Jun  MM  created
c

C==============
C Definitions
C==============

      implicit none

      character*3 PREFIX
c     Prefix for IMACS image labels
      parameter (PREFIX='ift')

      integer nargs,psplot,pgstat,refxfine,refyfine,refzfine,refzcoarse
      integer xfine,yfine,zfine,zcoarse,indexc
      integer imptr(1),ier,ierr,nimnew,nimref,i

      real gravmin,gravmax,refgravang,gravang,xshift,yshift
      real dx(9),dy(9),gra(9),ex(9),ey(9),xr,yr,xn,yn

      character*80 arg,filename,reffilename,filter,reffilter,output
      character    pfix*3

      logical reffound,newfound

c     PGPLOT subroutine declaration.
      integer pgopen,val

      pfix=PREFIX
      
      
C============
C Data points
C============

      gra(1)=130.2
      gra(2)=87.7
      gra(3)=42.7
      gra(4)=-2.3
      gra(5)=-47.3
      gra(6)=-92.3
      gra(7)=-137.3
      gra(8)=-182.3
      gra(9)=-222.3

!      do i=1,9
!         gra(i)=-47.34-gra(i)
!      end do

      dx(1)=54.
      dx(2)=28.5
      dx(3)=11.5
      dx(4)=-6.7
      dx(5)=-11.1
      dx(6)=-1.6
      dx(7)=27.75
      dx(8)=51.45
      dx(9)=53.5

      ex(1)=5.
      ex(2)=6.
      ex(3)=5.
      ex(4)=6.
      ex(5)=6.
      ex(6)=3.
      ex(7)=10.
      ex(8)=6.
      ex(9)=7.

      dy(1)=63.3
      dy(2)=22
      dy(3)=6.8
      dy(4)=5.5
      dy(5)=41.1
      dy(6)=67.5
      dy(7)=60.
      dy(8)=60.8
      dy(9)=64.

      ey(1)=11.
      ey(2)=13.
      ey(3)=11.
      ey(4)=7.
      ey(5)=15.
      ey(6)=16.
      ey(7)=9.
      ey(8)=6.
      ey(9)=9.


C=========================
C Set initial parameters 
C=========================

C     Get command-line arguments.
 10   nargs = iargc()
      if (nargs .lt. 2) then
         write (*,'(3(a))') "Usage: xydrift",
     $        " (reference file no., default prefix assumed)"
     $        ," (new file no.)"
         stop
      end if
      call getarg(1,arg)
      read (arg,*) nimref
      call getarg(2,arg)
      read (arg,*) nimnew


C============================================
C Get header info from reference & new images
C============================================

c     Get information from header
c     - Fabry-Perot parameters: X_fine, Y_fine, Z_fine, and Z_coarse
c     - Gravity angle
c     - Filter
c      
      write (reffilename,'(a3,i4.4)') prefix,nimref
      call openmosaicimage(reffilename,1,imptr,ierr)
      call imgkwi(imptr(1),"MMTF-X",refxfine,ier)
      call imgkwi(imptr(1),"MMTF-Y",refyfine,ier)
      call imgkwi(imptr(1),"MMTF-Z",refzfine,ier)
      call imgkwi(imptr(1),"MMTF-CZ",refzcoarse,ier)
      call imgkwr(imptr(1),"GRAV-MIN",gravmin,ier)
      call imgkwr(imptr(1),"GRAV-MAX",gravmax,ier)
      call imgkwc(imptr(1),"FILTER",reffilter,ier)
      refgravang=(gravmin+gravmax)/2.0
      call closemosaicimage(1,imptr,ierr)

      write (filename,'(a3,i4.4)') prefix,nimnew
      call openmosaicimage(filename,1,imptr,ierr)
      call imgkwi(imptr(1),"MMTF-X",xfine,ier)
      call imgkwi(imptr(1),"MMTF-Y",yfine,ier)
      call imgkwi(imptr(1),"MMTF-Z",zfine,ier)
      call imgkwi(imptr(1),"MMTF-CZ",zcoarse,ier)
      call imgkwr(imptr(1),"GRAV-MIN",gravmin,ier)
      call imgkwr(imptr(1),"GRAV-MAX",gravmax,ier)
      call imgkwc(imptr(1),"FILTER",filter,ier)
      gravang=(gravmin+gravmax)/2.0
      call closemosaicimage(1,imptr,ierr)


C===================
C Output header info
C===================
      

      print *, ''
      write (6, '("       ",a7,"   ",a7)') reffilename,filename
      print *, '       (Ref)     (New)'
      print *, '      =======   ======='
      write (6, '("X_f:      ",i4.4,"      ",i4.4)') refxfine, xfine
      write (6, '("Y_f:      ",i4.4,"      ",i4.4)') refyfine, yfine
      write (6, '("Z_f:      ",i4.4,"      ",i4.4)') refzfine, zfine
      write (6, '("Grav:   ",f6.1,"    ",f6.1)') refgravang,gravang
      print *, ''


C========================================================
C Check that reference and input have same Z_c and filter
C========================================================

      if (zcoarse.ne.refzcoarse) then
         print *, '*** z-coarse has changed since reference ***'
         print *, ' *** Cannot use as reference. Aborting. ***'
         goto 999
      end if

      if (filter(1:20).ne.reffilter(1:20)) then
         print *, '*** Filter has changed since reference ***'
         print *, ' ** Cannot use as reference. Aborting. **'
         goto 999
      end if

c     Plot results to screen
      psplot = 0
C     Look for proper screen device
 70   pgstat = pgopen('/XS')
      if (pgstat .le. 0) then
         write(*,*) "/XS device unavailable; trying /AQT."
         pgstat = pgopen('/AQT')
         if (pgstat .le. 0) then
            write(*,*) "/AQT device unavailable; querying."
            pgstat = pgopen('?')
         end if
      end if


      ! Initialize !
      call pgscf(2)
      call pgslw(2)
      
      ! Draw axes and labels !
      call pgsvp(0.1,0.9,0.1,0.5)
      call pgswin(-250.0,150.0,-25.0,90.0)
      call pgbox('BCNST',0,0,'BCNST',0,0)
      call pglabel ('Gravity Angle [deg]','\gDY','')
      
      ! Draw the Danger Zone !
      call pgqci(indexc)
      call pgsci(2)
      call pgsfs(2)
      call pgrect(-125.,-15.,-20.,85.)
      call pgsfs(3)
      call pgrect(-125.,-15.,-20.,85.)
      call pgsci(indexc)

      if (refgravang.gt.-125 .and. refgravang .lt.-20) then
         print *, '               *** WARNING ***'
         print *, '  Your reference frame is in the danger zone!'
         print *, 'You may want to use another frame as reference.'
         print *, '                 *********** '
         print *, ''
      end if

      if (gravang.gt.-125 .and. gravang .lt.-20) then
         print *, '               *** WARNING ***'
         print *, 'The current gravity angle is in the danger zone!'
         print *, ' The estimate of dY will have high uncertainty'
         print *, '                 *********** '
         print *, ''
      end if

      ! Draw new and reference gravity angles !
      call pgqci(indexc)
      call pgslw(20)
      call pgsci(8)
      call pgmove(refgravang,100.)
      call pgdraw(refgravang,-100.)
      call pgslw(10)
      call pgsci(11)
      call pgmove(gravang,100.)
      call pgdraw(gravang,-100.)
      call pgsci(indexc)
      call pgslw(1)

      ! Draw points !
      call pgpnts(9,gra,dy,17,1)
      call pgerrb(6,9,gra,dy,ey,1.0)
      
      ! Now, do DX part !
      ! ================!
      
      ! Draw axes and labels !
      call pgsvp(0.1,0.9,0.5,0.9)      
      call pgswin(-250.0,150.0,-25.0,75.0)
      call pgbox('BCST',0,0,'BCNST',0,0)
      call pglabel ('','\gDX','')

      ! Draw new and reference gravity angles !
      call pgqci(indexc)
      call pgslw(20)
      call pgsci(8)
      call pgmove(refgravang,100.)
      call pgdraw(refgravang,-100.)
      call pgslw(10)
      call pgsci(11)
      call pgmove(gravang,100.)
      call pgdraw(gravang,-100.)
      call pgsci(indexc)
      call pgslw(1)
      
      ! Draw points !
      call pgpnts(9,gra,dx,17,1)
      call pgerrb(6,9,gra,dx,ex,1.0)

      ! Now, calculate and output changes !

      reffound=.false.
      newfound=.false.
      do i=1,8
         if (gra(i).gt.refgravang .and. gra(i+1).lt.refgravang) then
            xr=((dx(i+1)-dx(i))/(gra(i+1)-gra(i)))*(refgravang-gra(i))
            xr=xr + dx(i)
            yr=((dy(i+1)-dy(i))/(gra(i+1)-gra(i)))*(refgravang-gra(i))
            yr=yr + dy(i)
            reffound=.true.
         end if
         if (gra(i).gt.gravang .and. gra(i+1).lt.gravang) then
            xn=((dx(i+1)-dx(i))/(gra(i+1)-gra(i)))*(gravang-gra(i))
            xn=xn + dx(i)
            yn=((dy(i+1)-dy(i))/(gra(i+1)-gra(i)))*(gravang-gra(i))
            yn=yn + dy(i)
            newfound=.true.
         end if
      end do

      if (.not. reffound) then
         if (refgravang.lt.gra(9)) then
            xr=dx(9)
            yr=dy(9)
         else if (refgravang.gt.gra(1)) then
            xr=dx(1)
            yr=dy(9)
         else
            print *, 'Gravity angle in reference frame is wierd...'
            print *, 'Shutting down...'
            goto 999
         end if
      end if
      if (.not.newfound) then
         if (gravang.lt.gra(9)) then
            xn=dx(9)
            yn=dy(9)
         else if (gravang.gt.gra(1)) then
            xn=dx(1)
            yn=dy(9)
         else
            print *, 'Gravity angle in new frame is wierd...'
            print *, 'Shutting down...'
            goto 999
         end if
      end if

      xshift=xn-xr
      yshift=yn-yr


    ! Draw shifts horizontally !
      call pgslw(1)
      ! DY !
      call pgsvp(0.1,0.9,0.1,0.5)
      call pgswin(-250.0,150.0,-25.0,90.0)
      call pgsci(8)
      call pgmove(-250.,yr)
      call pgdraw(refgravang,yr)
      call pgsci(11)
      call pgmove(-250.,yn)
      call pgdraw(gravang,yn)
      ! DX !
      call pgsvp(0.1,0.9,0.5,0.9)      
      call pgswin(-250.0,150.0,-25.0,75.0)
      call pgsci(8)
      call pgmove(-250.,xr)
      call pgdraw(refgravang,xr)
      call pgsci(11)
      call pgmove(-250.,xn)
      call pgdraw(gravang,xn)

      call pgsci(indexc)
  

      ! Output results to terminal !
      if (xshift.lt.0) then
         write (6,'("X_new = X_ref - ",f4.0)') abs(xshift)
      else
         write (6,'("X_new = X_ref + ",f4.0)') xshift
      end if
      if (yshift.lt.0) then
         write (6,'("Y_new = Y_ref - ",f4.0)') abs(yshift)
      else
         write (6,'("Y_new = Y_ref + ",f4.0)') yshift
      end if

      print *, ''


      ! Do upper labels !

      call pgsci(8)
      write (output,'("Reference angle = ",f6.1)') refgravang
      call pgmtext('T',3.0,0.25,1.0,output)
      call pgsci(11)
      write (output,'("New angle = ",f6.1)') gravang
      call pgmtext('T',1.9,0.25,1.0,output)
      call pgsci(indexc)


      if (xshift.lt.0) then
         write (output,'("X\dnew\u = X\dref\u - ",f5.1)') abs(xshift)
      else
         write (output,'("X\dnew\u = X\dref\u + ",f5.1)') xshift
      end if
      call pgmtext('T',3.0,0.35,0.0,output)
      if (yshift.lt.0) then
         write (output,'("Y\dnew\u = Y\dref\u - ",f5.1)') abs(yshift)
      else
         write (output,'("Y\dnew\u = Y\dref\u + ",f5.1)') yshift
      end if
      call pgmtext('T',1.9,0.35,0.0,output)

      call pgmtext('T',3.0,0.65,0.0,'Reference Frame: ')
      call pgmtext('T',1.9,0.65,0.0,'New Frame: ')

      call pgmtext('T',3.0,0.98,1.0,reffilename)
      call pgmtext('T',1.9,0.98,1.0,filename)

      
C     THE END
 999  end
